//MAP DATA
var mapTilesData = require('./config/MapTilesData');
var mapObjectsData = require('./config/MapObjectsData');
var mapItemsData = require('./config/MapItemsData');

//MODELS
var Player = require('./models/Player');
var Bullet = require('./models/Bullet');
var MapTiles = require('./models/MapTiles');
var MapObjects = require('./models/MapObjects');
var MapItems = require('./models/MapItems');
var Projectiles = require('./models/Projectiles');

//HELPERS/DEFAULTS
var MapObjectFactory = require('./utils/MapObjectFactory');
var MapItemFactory = require('./utils/MapItemFactory');
var spawnPoints = require('./utils/spawnPoints');

var Keys = require('./utils/keys');
var assets = require('./config/assets');
var engine = require('./utils/emitter');
var getDirection = require('./utils/getDirection');
var getNumbers = require('./utils/getNumbers');
// GLOBALS
var players = {}, map, projectiles, mapObjects, bullets, startTime, raf, ctx, canvas = {}, assets;

// engineEmitter.on('player move', () => {});
// engineEmitter.on('player dead', () => {});
// engineEmitter.on('bullet fired', () => {});
// engineEmitter.on('bullet hit', () => {});

var newPlayer = function (playerID) {
  let currentNumbers = getNumbers(players);
  let newNumber;
  for (let i = 1; i <= 4; i++) {
    if (currentNumbers.indexOf(i) === -1) {
      newNumber = i;
      break;
    }
  }
  console.log('player added, number:', newNumber, 'id:', playerID)
  if (newNumber) {
    let position = spawnPoints[newNumber];
    let newPlayer = new Player(position.x, position.y);
    newPlayer.id = playerID;
    newPlayer.number = newNumber;
    players[playerID] = newPlayer;
    engine.emitter.emit('update players', players);
  }
}
var removePlayer = function (playerID) {
  // let position = -1;
  if (players[playerID]) {
    delete players[playerID];
  }
  engine.emitter.emit('update players', players);
}

var playerMoved = function (player) {
  players[player.id].x = player.x;
  players[player.id].y = player.y;
  players[player.id].hsp = player.hsp;
  players[player.id].vsp = player.vsp;
  engine.emitter.emit('update player', players[player.id])
}

var bulletFired = function (bullet) {
  let args = {
    player: {
      x: bullet.x,
      y: bullet.y
    },
    speed: {
      hsp: bullet.hsp,
      vsp: bullet.vsp
    }
  }
  var newBullet = players[bullet.owner.id].fireBullet(getDirection(args.speed))
  bullets.items.push(newBullet);
  engine.emitter.emit('create bullet', newBullet);
  engine.emitter.emit('update players', players)
}

var weaponRemoved = function (data) {
  mapItems.weapons[data.weapon.id].active = false;
  mapItems.weapons[data.weapon.id].restock();
  players[data.player.id].giveWeapon(data.weapon);
  engine.emitter.emit('remove weapon', data.weapon)
  engine.emitter.emit('update players', players)
}

var itemRemoved = function (data) {
  mapItems.items[data.item.id].active = false;
  mapItems.items[data.item.id].restock();
  players[data.player.id].giveItem(data.item);
  engine.emitter.emit('remove item', data.item)
  engine.emitter.emit('update players', players)
}

var playerTakeDamage = function (data) {
  players[data.player.id].takeDamage(data.value);
  engine.emitter.emit('on player take damage', data)
  engine.emitter.emit('update players', players)
}

var playerDeath = function (data) {
  // delete players[data.player.id]
  let newPosition = spawnPoints[Math.ceil(Math.random() * 4)];
  players[data.player.id].playerDeath(data.enemy, players);
  engine.emitter.emit('on player death', {
    player: data.player,
    enemy: data.enemy,
    newX: newPosition.x,
    newY: newPosition.y
  });
  players[data.player.id].health = 4;
  engine.emitter.emit('update players', players);
}

engine.emitter.on('new player', newPlayer);
engine.emitter.on('remove player', removePlayer);
engine.emitter.on('player moved', playerMoved);
engine.emitter.on('bullet fired', bulletFired);
engine.emitter.on('weapon picked up', weaponRemoved);
engine.emitter.on('item picked up', itemRemoved);
engine.emitter.on('player take damage', playerTakeDamage);
engine.emitter.on('player has died', playerDeath);

//SETUP
var setup = (canvasWidth, canvasHeight, reqAnimFrame, context, spriteUrls) => {
  ctx = context;
  startTime = Date.now();
  canvas.height = canvasHeight;
  canvas.width = canvasWidth;
  var mapObjectMappings = {
    11: MapObjectFactory.wall,
    77: MapObjectFactory.dynamic["lava"],
    88: MapObjectFactory.dynamic["ice"],
    99: MapObjectFactory.dynamic["mud"]
  }
  mapObjects = new MapObjects(mapObjectsData, mapObjectMappings);
  var mapItemMappings = {
    22: MapItemFactory.weapon["assault"],
    33: MapItemFactory.weapon["shotgun"],
    44: MapItemFactory.item["health"],
    55: MapItemFactory.item["overshield"]
  }
  mapItems = new MapItems(mapItemsData, mapItemMappings);
  bullets = new Projectiles();
  setInterval(main, 500);
};

// UPDATE - RUN MULTIPLE TIMES A SECOND!
var update = function() {
  console.log('updating, current # players', Object.keys(players).length);

  //UPDATE BULLETS X/Y && DELETE WHEN OFF MAP
  // bulletUpdate();
  bullets.items.forEach(function (bullet) {
    bullet.move()
  });
  //DESTROY BULLETS THAT ARE OFF CANVAS
  bullets.detectOffCanvas(canvas.height, canvas.width);
  //DESTROY BULLETS THAT IMPACT WITH mapObjects
  bullets.detectWallCollisions(mapObjects.collideable);
}

//GAME LOOP
var main = function () {
  var now = Date.now();
  var delta = now - startTime;
  update();
  startTime = now;
};

module.exports.setup = setup;
module.exports.players = players;
module.exports.emitter = engine.emitter;
module.exports.createListener = engine.createListener;